@@ -1,4 +1,4 @@
 (defpackage :test-asdf/monodll-user (:use)) ;; dummy, for package-inferred-system dependencies.
 
-(in-package :test-package)
+(in-package :asdf-test)
 (ffi:def-function "always_42" () :returning :int)
