@@ -632,7 +632,15 @@ is bound, write a message and exit on an error.  If
            `(:output-translations
              (,(acall :wilden *asdf-directory*) ,(acall :wilden (resolve-output "asdf/")))
              (t ,(acall :wilden (resolve-output "root")))
-             :ignore-inherited-configuration)))
+             :ignore-inherited-configuration))
+
+    (let ((fasl-dir (resolve-output "asdf/test/")))
+      (when (acall (list '#:directory-exists-p :uiop) fasl-dir)
+        (format t "Removing old fasls from directory ~a.~%" fasl-dir)
+        (funcall (asym '#:delete-directory-tree :uiop t)
+                 fasl-dir
+                 :validate #'(lambda (x)
+                               (member "build" (pathname-directory x) :test 'equal))))))
   (when (asym :*central-registry*)
     (set (asym :*central-registry*) `(,*test-directory*)))
   (format t "Being a bit verbose~%")
@@ -761,6 +769,10 @@ is bound, write a message and exit on an error.  If
     (format t "Now loading new asdf via method ~A~%" new-method)
     (acall (list new-method :asdf-test))
     (format t "Testing it~%")
+    (format t "UIOP: ~S ~S~%" (asymval :*uiop-version*) (acall :system-version (acall :find-system :uiop)))
+    (format t "ASDF: ~S ~S~%" (get-asdf-version) (acall :system-version (acall :find-system :asdf)))
+    (assert (equal (asymval :*uiop-version*) (acall :system-version (acall :find-system :uiop))))
+    (assert (equal (get-asdf-version) (acall :system-version (acall :find-system :asdf))))
     (register-directory *test-directory*)
     (load-test-system :test-asdf/upgrade)
     (assert (symbol-value '*properly-upgraded*))))
